##
## test_root.py
## Login : <uli@pu.smp.net>
## Started on  Thu Jan 20 09:26:54 2011 Uli Fouquet
## $Id$
## 
## Copyright (C) 2011 Uli Fouquet
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Test applicants root.
"""
import logging
import shutil
import tempfile
import unittest
from StringIO import StringIO
from zope.app.testing.functional import FunctionalTestCase
from zope.component.hooks import setSite, clearSite
from zope.interface.verify import verifyClass, verifyObject
from zope.site import LocalSiteManager
from waeup.sirp.app import University
from waeup.sirp.applicants import (
    interfaces, application_exists, get_applicant_data, Applicant,
    ApplicantsContainer,
    )
from waeup.sirp.applicants.root import (
    ApplicantsRoot, ApplicantsPlugin,
    )
from waeup.sirp.testing import FunctionalLayer




class FakeSite(dict):
    pass

class ApplicantsRootTestCase(FunctionalTestCase):

    layer = FunctionalLayer

    def setUp(self):
        super(ApplicantsRootTestCase, self).setUp()

        # Setup a sample site for each test
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        self.app = self.getRootFolder()['app']
        setSite(self.app)
        return

    def tearDown(self):
        super(ApplicantsRootTestCase, self).tearDown()
        shutil.rmtree(self.dc_root)
        return

    def test_interfaces(self):
        # Make sure the correct interfaces are implemented.
        self.assertTrue(
            verifyClass(
                interfaces.IApplicantsRoot, ApplicantsRoot)
            )
        self.assertTrue(
            verifyObject(
                interfaces.IApplicantsRoot, ApplicantsRoot())
            )
        return

    def test_logger(self):
        # We can get a logger from root
        logger = self.app['applicants'].logger
        assert logger is not None
        assert logger.name == 'waeup.sirp.app.applicants'
        handlers = logger.handlers
        assert len(handlers) == 1
        filename = logger.handlers[0].baseFilename
        assert filename.endswith('applicants.log')
        assert filename.startswith(self.dc_root)

class ApplicantsRootPluginTestCase(unittest.TestCase):
    def create_logger(self):
        # create a logger suitable for local tests.
        test_logger = logging.getLogger('waeup.sirp.applicants.testlogger')
        log = StringIO()
        handler = logging.StreamHandler(log)
        handler.setLevel(logging.DEBUG)
        test_logger.addHandler(handler)
        test_logger.setLevel(logging.DEBUG)
        self.logger = test_logger
        self.log = log
        self.handler = handler
        return self.logger

    def remove_logger(self):
        del self.handler
        del self.logger
        del self.log
        pass

    def get_log(self):
        self.log.seek(0)
        return self.log.read()

    def setUp(self):
        self.create_logger()
        return

    def tearDown(self):
        self.remove_logger()
        return

    # Real tests start here...
    def test_pluginsetup(self):
        # Make sure we can add ApplicantsRoot to sites.
        site = FakeSite()
        plugin = ApplicantsPlugin()
        plugin.setup(site, 'blah', self.logger)
        self.assertTrue('applicants' in site.keys())
        log = self.get_log()
        self.assertTrue('Installed applicants root.' in log)
        return

    def test_update_new(self):
        # Run update on a site without applicants root.
        site = FakeSite()
        plugin = ApplicantsPlugin()
        plugin.update(site, 'blah', self.logger)
        self.assertTrue('applicants' in site.keys())
        log = self.get_log()
        self.assertTrue('Updating site at <Unnamed Site>' in log)
        self.assertTrue('Installed applicants root.' in log)
        return

    def test_update_outdated(self):
        # Run update on a site with outdated applicants root.
        site = FakeSite()
        root = object() # # This is not a proper applicants root
        site['applicants'] = root
        plugin = ApplicantsPlugin()
        plugin.update(site, 'blah', self.logger)
        self.assertTrue(site['applicants'] is not root)
        self.assertTrue(isinstance(site['applicants'], ApplicantsRoot))
        log = self.get_log()
        self.assertTrue('Outdated applicants folder detected' in log)
        self.assertTrue('Updating site at <Unnamed Site>' in log)
        self.assertTrue('Installed applicants root.' in log)
        return

    def test_update_uptodate(self):
        # Run update on a site with proper applicants root.
        site = FakeSite()
        root = ApplicantsRoot()
        site['applicants'] = root
        plugin = ApplicantsPlugin()
        plugin.update(site, 'blah', self.logger)
        self.assertTrue(site['applicants'] is root)
        log = self.get_log()
        self.assertTrue('Updating site at <Unnamed Site>' in log)
        self.assertTrue('Nothing to do' in log)
        return

    def test_update_log(self):
        # Check that sitename is used in log messages on updates.
        site = FakeSite()
        site.__name__ = 'my_site'
        plugin = ApplicantsPlugin()
        plugin.update(site, 'blah', self.logger)
        log = self.get_log()
        self.assertTrue('Updating site at my_site.' in log)
        return


class HelperToolsTest(FunctionalTestCase):

    layer = FunctionalLayer

    def setUp(self):
        super(HelperToolsTest, self).setUp()

        # Setup a sample site for each test
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        self.app = self.getRootFolder()['app']
        setSite(self.app)

        # Insert some applicants containers and applicants...
        self.container1 = ApplicantsContainer()
        self.app['applicants']['foo'] = self.container1
        self.container2 = ApplicantsContainer()
        self.app['applicants']['bar'] = self.container2
        self.ac = u'APP-99999'
        self.applicant = Applicant()
        self.applicant.access_code = self.ac
        self.app['applicants']['foo'][self.ac] = self.applicant
        return

    def tearDown(self):
        super(HelperToolsTest, self).tearDown()
        shutil.rmtree(self.dc_root)
        return

    def test_application_exists(self):
        result = application_exists('APP-99999')
        assert result is True

        result = application_exists('APP-44444')
        assert result is False
        return

    def test_get_applicant_data(self):
        result = get_applicant_data('APP-99999')
        self.assertEqual(result, self.applicant)
        return

    def test_get_applicant_data_none(self):
        result = get_applicant_data('NOT-EXISTIING')
        assert result is None
        return

def suite():
    suite = unittest.TestSuite()
    for testcase in [
            ApplicantsRootTestCase,
            ApplicantsRootPluginTestCase,
            HelperToolsTest,
            ]:
        suite.addTests(unittest.makeSuite(testcase))
    return suite

test_suite = suite
